// Oleg Petruny proprietary.


#include "CustomGameInstanceBase.h"

#include "EnhancedInputLibrary.h"
#include "EnhancedInputSubsystems.h"
#include "InputMappingContext.h"
#include "Kismet/GameplayStatics.h"

#include "CustomGameUserSettings.h"
#include "Interactable/Activators/InCameraInteractableActivator.h"
#include "Interactable/Activators/RaycastInteractableActivator.h"
#include "Interactable/Modificators/ActivateInteractableModificator.h"
#include "Interactable/Modificators/SawInteractableModificator.h"

void UCustomGameInstanceBase::Init()
{
    UGameInstance::Init();

    // IN FUTURE ASSIGN FROM CONTENT LOADER MEMBER
    interactionsActivators.Add(URaycastInteractableActivator::StaticClass());
    interactionsActivators.Add(UInCameraInteractableActivator::StaticClass());

    for(auto& modificator : interactionsModificators)
    {
        if(modificator.GetDefaultObject()->GetMappingContext())
        {
            inputContexts.Add(modificator.GetDefaultObject()->GetMappingContext());
        }
    }

    ApplyMouseSettings();
}

void UCustomGameInstanceBase::ApplyMouseSettings()
{
    if(auto gameSettings = UCustomGameUserSettings::GetCustomGameUserSettings())
    {
        for(auto& context : inputContexts)
        {
            if(!context.LoadSynchronous())
                continue;

            for(auto& mapping : context.LoadSynchronous()->GetMappings())
            {
                if(mapping.Key == EKeys::Mouse2D)
                {
                    for(auto& modifier : mapping.Modifiers)
                    {
                        if(auto negate_modifier = Cast<UInputModifierNegate>(modifier))
                        {
                            negate_modifier->bY = !gameSettings->bMouseInverted;
                        }
                        if(auto scalar_modifier = Cast<UInputModifierScalar>(modifier))
                        {
                            scalar_modifier->Scalar = FVector{ gameSettings->GetMouseSensetivity() * 0.5 };
                        }
                    }
                }
            }
            UEnhancedInputLibrary::RequestRebuildControlMappingsUsingContext(context.LoadSynchronous());
        }
    }
}

void UCustomGameInstanceBase::AppendInteractableModificatorClass(TSubclassOf<class UInteractableModificator> modificator)
{
    bool alreadyPresent = false;
    interactionsModificators.Add(modificator, &alreadyPresent);
    if(!alreadyPresent)
    {
        if(auto IC = modificator.GetDefaultObject()->GetMappingContext())
        {
            inputContexts.Add(IC);
            if(auto PC = UGameplayStatics::GetPlayerController(GetWorld(), 0))
            {
                if(auto inputSubsystem = ULocalPlayer::GetSubsystem<UEnhancedInputLocalPlayerSubsystem>(PC->GetLocalPlayer()))
                {
                    inputSubsystem->AddMappingContext(IC, 0);
                }
            }
        }
    }
}