// Oleg Petruny proprietary.


#include "Interactable.h"

#include "Kismet/GameplayStatics.h"

#include "../MainGameModeBase.h"
#include "../PlayerBase.h"
#include "../Widgets/WidgetsManager.h"
#include "Modificators/InteractableModificator.h"

int32 AInteractable::GetActivatedFlags()
{
    return activated;
}

void AInteractable::BeginPlay()
{
    AActor::BeginPlay();

    // FOR ADRESS TO COLOR
    //if(auto m = material.LoadSynchronous())
    //{
    //    if(material->VectorParameterValues.Num() > 0)
    //    {
    //        GEngine->AddOnScreenDebugMessage(5, 5.0f, FColor::Emerald, material->VectorParameterValues[0].ParameterInfo.Name.ToString());
    //    }
    //}

    TArray<UInteractableModificator*> instancedModificators;
    GetComponents(instancedModificators);

    uint8 activatorTypes = 0;
    for(auto& modificator : instancedModificators)
    {
        uint8 modificatorActivatorTypes = static_cast<uint8>(modificator->GetActivatorTypes());
        activatorTypes |= modificatorActivatorTypes;

        switch(modificatorActivatorTypes & static_cast<uint8>(EActivatorType::Use))
        {
            case static_cast<uint8>(EActivatorType::Use):
                modificators.Add({ EActivatorType::Use, modificator });
                break;
            default:
                break;
        }
        switch(modificatorActivatorTypes & static_cast<uint8>(EActivatorType::Saw))
        {
            case static_cast<uint8>(EActivatorType::Saw):
                modificators.Add({ EActivatorType::Saw, modificator });
                break;
            default:
                break;
        }
    }

    TArray<UMeshComponent*> meshes;
    GetComponents(meshes);
    for(auto mesh : meshes)
    {
        if(activatorTypes)
        {
            mesh->SetCollisionProfileName(TEXT("Interactable"));
        }

        if(activatorTypes & static_cast<uint8>(EActivatorType::Saw))
        {
            mesh->CustomDepthStencilValue = 128;
            mesh->CustomDepthStencilWriteMask = ERendererStencilMask::ERSM_Default;
            mesh->SetRenderCustomDepth(true);
        }
    }

    if(auto PC = UGameplayStatics::GetPlayerController(GetWorld(), 0))
        player = Cast<APlayerBase>(PC->GetPawn());
}

void AInteractable::_Activate(EActivatorType type)
{
#ifdef INTERACTABLE_DEBUG
    GEngine->AddOnScreenDebugMessage(30 + (int)type, 5.0f, FColor::Cyan, TEXT("Player activate: ") + this->GetName()
        + TEXT(" by: ") + StaticEnum<EActivatorType>()->GetNameByValue(static_cast<int64>(type)).ToString());
#endif // INTERACTABLE_DEBUG

    if(!player)
        return;

    if(modificators.Num() == 0)
        return;

    activated |= static_cast<uint8>(type);
    if(auto WM = AMainGameModeBase::GetWidgetManager())
    {
        for(const auto& modificator : modificators)
        {
            if(static_cast<uint8>(modificator.Value->GetActivatorTypes()) & static_cast<uint8>(type))
            {
                WM->ShowInteractionHints(modificator.Value);
                modificator.Value->Bind_Implementation(player->inputComponent);
            }
        }
    }

    Activate_Implementation(type);
}

void AInteractable::_Deactivate(EActivatorType type)
{
#ifdef INTERACTABLE_DEBUG
    GEngine->AddOnScreenDebugMessage(30 + (int)type, 5.0f, FColor::Magenta, TEXT("Player deactivate: ") + this->GetName()
        + TEXT(" by: ") + StaticEnum<EActivatorType>()->GetNameByValue(static_cast<int64>(type)).ToString());
#endif // INTERACTABLE_DEBUG

    if(!player)
        return;

    if(modificators.Num() == 0)
        return;

    activated &= ~static_cast<uint8>(type);
    if(auto WM = AMainGameModeBase::GetWidgetManager())
    {
        WM->HideInteractionHints(type);
        for(const auto& modificator : modificators)
        {
            if(static_cast<uint8>(modificator.Value->GetActivatorTypes()) & static_cast<uint8>(type))
            {
                modificator.Value->Unbind_Implementation();
            }
        }
    }

    Deactivate_Implementation(type);
}
