// Oleg Petruny proprietary.

#pragma once

#include "CoreMinimal.h"
#include "InputActionValue.h"
#include "InteractableModificator.h"

#include "MoveInteractableModificator.generated.h"

DECLARE_DYNAMIC_MULTICAST_DELEGATE(FActivateMoveInteractableModificatorActivatedDelegate);
DECLARE_DYNAMIC_MULTICAST_DELEGATE(FDeactivateMoveInteractableModificatorActivatedDelegate);
DECLARE_DYNAMIC_MULTICAST_DELEGATE(FHoldingMoveInteractableModificatorActivatedDelegate);
DECLARE_DYNAMIC_MULTICAST_DELEGATE(FRotatingMoveInteractableModificatorActivatedDelegate);

UCLASS(ClassGroup = InteractableModificator, meta = (BlueprintSpawnableComponent), Blueprintable, BlueprintType)
class UMoveInteractableModificator : public UInteractableModificator
{
    GENERATED_BODY()

public:
    UMoveInteractableModificator(const FObjectInitializer& ObjectInitializer);
    virtual void TickComponent(float DeltaTime, enum ELevelTick TickType, FActorComponentTickFunction* ThisTickFunction) override;


    void Bind_Implementation(class UEnhancedInputComponent* input) override;
    void Unbind_Implementation() override;

    UPROPERTY(BlueprintAssignable)
    FActivateMoveInteractableModificatorActivatedDelegate OnMoveActivated;
    UPROPERTY(BlueprintAssignable)
    FDeactivateMoveInteractableModificatorActivatedDelegate OnMoveDeactivated;
    UPROPERTY(BlueprintAssignable)
    FHoldingMoveInteractableModificatorActivatedDelegate OnHolding;
    UPROPERTY(BlueprintAssignable)
    FRotatingMoveInteractableModificatorActivatedDelegate OnRotating;

protected:
    float distance = 0;
    float minDistance = 100.0f;
    float maxDistance = 200.0f;
    float zoomSpeed = 10.0f;
    float moveSpeed = 1.0f;
    float rotateSpeed = 10.0f;

private:
    void IncreaseState();
    void DecreaseState();
    void ProcessState();
    void UpdatePosition(float deltaTime);
    void Zooming(const FInputActionValue& value);
    void Rotating(const FInputActionValue& axis);

    enum class State : uint8
    {
        Nothing = 0,
        Holding,
        Rotating
    };

    class UEnhancedInputComponent* lastInput;
    TArray<int32> bindindingHandlers;
    State state = State::Nothing;
    class AInteractable* actor;
    class APlayerBase* player;
};
